!
!  GKSTEST1.f90 
!
!  FUNCTIONS:
!	GKSTEST1      - Entry point of console application.
!
!
!****************************************************************************
!
!  PROGRAM: GKSTEST1
!
!  PURPOSE:  First code with GKS
!
!****************************************************************************

program GKSTEST1

	implicit none

! Physical Variables
	real :: gamma
	real :: step,peak,erf
	real :: LD,dx,x,TD,dt,t
	integer :: Nx,Nt
	
! Conserved Variables
	real,dimension(:),allocatable :: rho0
	real,dimension(:),allocatable :: mom0
	real,dimension(:),allocatable :: E0

	real,dimension(:),allocatable :: rho1 
	real,dimension(:),allocatable :: mom1
	real,dimension(:),allocatable :: E1

! Generic Variables
	integer :: i,j,k,m,nWW,nW,nE,nEE

	real,dimension(1:3) :: hNVP,hNVE,hNVEE,hNVW,hNVWW

	real,dimension(1:3) :: hIVL,hIVR
	real,dimension(1:3) :: hGIVL,hGIVR

	real,dimension(1:3) :: hIVLp,hIVLn,hIVRp,hIVRn
	real,dimension(1:3) :: hGIVLp,hGIVLn,hGIVRp,hGIVRn
	real,dimension(1:3) :: dummy1,dummy2

	real,dimension(1:3) :: dp,ap,bp
	real,dimension(1:3,1:3) :: Rp
	real,dimension(1:3) :: dn,an,bn
	real,dimension(1:3,1:3) :: Rn
	real,dimension(1:3) :: atot,btot
	real,dimension(1:3,1:3) :: Rtot

	real,dimension(1:3) :: Flux0L
	real,dimension(1:3) :: Flux1L
	real,dimension(1:3,1:3) :: FluxMatrix0L
	real,dimension(1:3) :: Flux0R
	real,dimension(1:3) :: Flux1R
	real,dimension(1:3,1:3) :: FluxMatrix0R

!============================================================================
! Input Data
!============================================================================

! Specific heat ratio
	gamma = 5.0/3.0

! Size of computational domain	
	dx = 0.02
	LD = 50.0*dx ! 100
	Nx = nint(LD/dx)

	dt = 0.005
	TD = 50.0*dt ! 250
	Nt = nint(TD/dt)

! Allocate memory
	allocate(rho0(1:Nx))
	allocate(mom0(1:Nx))
	allocate(E0(1:Nx))

	allocate(rho1(1:Nx))
	allocate(mom1(1:Nx))
	allocate(E1(1:Nx))

! Inital Values
	x = 0.0
    do i = 1,Nx,1
		x = dx*real(i-1)

! Riemann problem (Sod's test case)
		if (x<(real(dx*Nx)/2.0)) then
			rho0(i) = (1.0)
	        mom0(i) = (1.0)*(0.0)
		    E0(i)   = 3.0*(1.0)/2.0+(1.0)*(0.0)**2/2
		else
			rho0(i) = (0.125)
	        mom0(i) = (0.125)*(0.0)
		    E0(i)   = 3.0*(0.1)/2.0+(0.125)*(0.0)**2/2
		endif

		m=nint(real(Nx)/2.0)
		rho0(m) = (rho0(1)+rho0(Nx))/2.0
		mom0(m) = (mom0(1)+mom0(Nx))/2.0
		E0(m)   = (E0(1)+E0(Nx))/2.0

		rho1(i) = rho0(i)
		mom1(i) = mom0(i)
		E1(i)   = E0(i)

	enddo

!============================================================================
! Solver
!============================================================================

	t = 0.0
    do j = 1,Nt,1
		t = dt*real(j-1)

		x = 0.0
		do i = 3,(Nx-2),1
			x = dx*real(i-1)

			nWW = i-2
			nW  = i-1
			nE  = i+1
			nEE = i+2

! Periodic Boundary Conditions
			if (nWW<=0) then
				nWW = nWW+Nx
			endif
			if (nW<=0) then
				nW = nW+Nx
			endif

			if (nEE>Nx) then
				nEE = nEE-Nx
			endif
			if (nE>Nx) then
				nE = nE-Nx
			endif

			hNVWW(1) = rho0(nWW)
			hNVWW(2) = mom0(nWW)
			hNVWW(3) = E0(nWW)
			
			hNVW(1) = rho0(nW)
			hNVW(2) = mom0(nW)
			hNVW(3) = E0(nW)

			hNVP(1) = rho0(i)
			hNVP(2) = mom0(i)
			hNVP(3) = E0(i)

			hNVE(1) = rho0(nE)
			hNVE(2) = mom0(nE)
			hNVE(3) = E0(nE)

			hNVEE(1) = rho0(nEE)
			hNVEE(2) = mom0(nEE)
			hNVEE(3) = E0(nEE)

		if (-1>0) then

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
! Lax Wendroff
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

! Interface Variables
			call ReconI(dx,hNVW,hNVP,hIVL,hGIVL)
			call ReconI(dx,hNVP,hNVE,hIVR,hGIVR)

! Fluxes
			call flux(hIVL(1),hIVL(2),hIVL(3),gamma,Flux0L)
			call matrixLW(hIVL(1),hIVL(2),hIVL(3),gamma,FluxMatrix0L)

			call flux(hIVR(1),hIVR(2),hIVR(3),gamma,Flux0R)
			call matrixLW(hIVR(1),hIVR(2),hIVR(3),gamma,FluxMatrix0R)

			do k = 1,3,1
				Flux1L(k) = dt*Flux0L(k)-dt**2.0/2.0*&
					(FluxMatrix0L(k,1)*hGIVL(1)+FluxMatrix0L(k,2)*hGIVL(2)+FluxMatrix0L(k,3)*hGIVL(3))
				Flux1R(k) = dt*Flux0R(k)-dt**2.0/2.0*&
					(FluxMatrix0R(k,1)*hGIVR(1)+FluxMatrix0R(k,2)*hGIVR(2)+FluxMatrix0R(k,3)*hGIVR(3))
			enddo

		else

!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$
!   GKS
!$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$

!	WW		|		W		|		P		|		E		|		EE
!			|			IVLn|IVLp		IVRn|IVRp			|		
!			|		   GIVLn|GIVLp	   GIVRn|GIVRp			|
!	--->  x

! Interface Variables

			call ReconII(dx,hNVWW,hNVW,hNVP,dummy1,hIVLn,dummy2,hGIVLn)
			call ReconII(dx,hNVW,hNVP,hNVE,hIVLp,hIVRn,hGIVLp,hGIVRn)
			call ReconII(dx,hNVP,hNVE,hNVEE,hIVRp,dummy1,hGIVRp,dummy2)

			if (1>0) then
	!	============ Left ============ 
	!			\xi	<---- (p)
				call vector_dn(hIVLp(1),hIVLp(2),hIVLp(3),dn)
	!			\xi	(n) ---->
				call vector_dp(hIVLn(1),hIVLn(2),hIVLn(3),dp)

				do k = 1,3,1
					hIVLn(k) = dn(k)+dp(k)
					hIVLp(k) = hIVLn(k)

					hGIVLn(k) = (hIVLn(k)-hNVW(k))/(dx/2.0)
					hGIVLp(k) = (hNVP(k)-hIVLp(k))/(dx/2.0)
				enddo

	!	============ Right ============ 
	!			\xi	<---- (p)
				call vector_dn(hIVRp(1),hIVRp(2),hIVRp(3),dn)
	!			\xi	(n) ---->
				call vector_dp(hIVRn(1),hIVRn(2),hIVRn(3),dp)

				do k = 1,3,1
					hIVRn(k) = dn(k)+dp(k)
					hIVRp(k) = hIVRn(k)

					hGIVRn(k) = (hIVRn(k)-hNVP(k))/(dx/2.0)
					hGIVRp(k) = (hNVE(k)-hIVRp(k))/(dx/2.0)
				enddo

			endif

! Fluxes
!	============ Left ============ 
!			\xi	<---- (p)
			call vector_an(hIVLp(1),hIVLp(2),hIVLp(3),an)
			call matrix_Rn(hIVLp(1),hIVLp(2),hIVLp(3),Rn)
			call vector_bn(hIVLp(1),hIVLp(2),hIVLp(3),hGIVLp(1),hGIVLp(2),hGIVLp(3),bn)

!			\xi	(n) ---->
			call vector_ap(hIVLn(1),hIVLn(2),hIVLn(3),ap)
			call matrix_Rp(hIVLn(1),hIVLn(2),hIVLn(3),Rp)
			call vector_bp(hIVLn(1),hIVLn(2),hIVLn(3),hGIVLn(1),hGIVLn(2),hGIVLn(3),bp)

			do k = 1,3,1
				atot(k) = an(k)+ap(k)
				btot(k) = bn(k)+bp(k)
				do m = 1,3,1
					Rtot(k,m) = Rn(k,m)+Rp(k,m)
				enddo
			enddo

			do k = 1,3,1
				Flux1L(k) = dt*(atot(k))-dt**2.0/2.0*&
					(Rtot(k,1)*btot(1)+Rtot(k,2)*btot(2)+Rtot(k,3)*btot(3))
			enddo

!	============ Right ============ 
!			\xi	<---- (p)
			call vector_an(hIVRp(1),hIVRp(2),hIVRp(3),an)
			call matrix_Rn(hIVRp(1),hIVRp(2),hIVRp(3),Rn)
			call vector_bn(hIVRp(1),hIVRp(2),hIVRp(3),hGIVRp(1),hGIVRp(2),hGIVRp(3),bn)

!			\xi	(n) ---->
			call vector_ap(hIVRn(1),hIVRn(2),hIVRn(3),ap)
			call matrix_Rp(hIVRn(1),hIVRn(2),hIVRn(3),Rp)
			call vector_bp(hIVRn(1),hIVRn(2),hIVRn(3),hGIVRn(1),hGIVRn(2),hGIVRn(3),bp)

			do k = 1,3,1
				atot(k) = an(k)+ap(k)
				btot(k) = bn(k)+bp(k)
				do m = 1,3,1
					Rtot(k,m) = Rn(k,m)+Rp(k,m)
				enddo
			enddo

			do k = 1,3,1
				Flux1R(k) = dt*(atot(k))-dt**2.0/2.0*&
					(Rtot(k,1)*btot(1)+Rtot(k,2)*btot(2)+Rtot(k,3)*btot(3))
			enddo

		endif

! Up-date
			rho1(i) = rho0(i)-(Flux1R(1)-Flux1L(1))/dx
			mom1(i) = mom0(i)-(Flux1R(2)-Flux1L(2))/dx
			E1(i)   = E0(i)  -(Flux1R(3)-Flux1L(3))/dx

		enddo

		do i = 1,Nx,1
! Switch		
			rho0(i) = rho1(i)
			mom0(i) = mom1(i)
			E0(i)   = E1(i)
		enddo
		
		print *,j

	enddo

!============================================================================
! Print Out
!============================================================================

	x = 0.0
	do i = 1,Nx,1
		x = dx*real(i-1)
		print *,x,rho0(i),mom0(i),E0(i)
	enddo

!%%%%%%%%%%%%
! Test 1
!%%%%%%%%%%%%
!
!	print *,' '
!	call vector_ap(1.23445,3.456676,5.7897,ap)
!	print *,ap(1),ap(2),ap(3)
!	call matrix_Rp(34.5678,5.789,4.3214,Rp)
!	print *,Rp(1,1),Rp(1,2),Rp(1,3)
!	print *,Rp(2,1),Rp(2,2),Rp(2,3)
!	print *,Rp(3,1),Rp(3,2),Rp(3,3)
!	call vector_bp(9.87654,3.142567,25.788,6.784,5.674,5.67787,bp)
!	print *,bp(1),bp(2),bp(3)
!	print *,' '
!	call vector_an(1.23445,3.456676,5.7897,an)
!	print *,an(1),an(2),an(3)
!	call matrix_Rn(34.5678,5.789,4.3214,Rn)
!	print *,Rn(1,1),Rn(1,2),Rn(1,3)
!	print *,Rn(2,1),Rn(2,2),Rn(2,3)
!	print *,Rn(3,1),Rn(3,2),Rn(3,3)
!	call vector_bn(9.87654,3.142567,25.788,6.784,5.674,5.67787,bn)
!	print *,bn(1),bn(2),bn(3)

!%%%%%%%%%%%%
! Test 2
!%%%%%%%%%%%%
!
!	dt = 0.23445
!	dx = 0.67899
	
!	hIVL(1) = 1.23445
!	hIVL(2) = 3.456676
!	hIVL(3) = 5.7897

!	hGIVL(1) = 34.5678
!	hGIVL(2) = 5.789
!	hGIVL(3) = 4.3214

!	call flux(hIVL(1),hIVL(2),hIVL(3),gamma,Flux0L)
!	call matrixLW(hIVL(1),hIVL(2),hIVL(3),gamma,FluxMatrix0L)

!   Lax-Wendroff
!	do k = 1,3,1
!		Flux1L(k) = dt*Flux0L(k)-dt**2.0/2.0*&
!			(FluxMatrix0L(k,1)*hGIVL(1)+FluxMatrix0L(k,2)*hGIVL(2)+FluxMatrix0L(k,3)*hGIVL(3))
!	enddo
!	print *,Flux1L(1),Flux1L(2),Flux1L(3)

!   GKS
!	call vector_an(hIVL(1),hIVL(2),hIVL(3),an)
!	call matrix_Rn(hIVL(1),hIVL(2),hIVL(3),Rn)
!	call vector_bn(hIVL(1),hIVL(2),hIVL(3),hGIVL(1),hGIVL(2),hGIVL(3),bn)

!	call vector_ap(hIVL(1),hIVL(2),hIVL(3),ap)
!	call matrix_Rp(hIVL(1),hIVL(2),hIVL(3),Rp)
!	call vector_bp(hIVL(1),hIVL(2),hIVL(3),hGIVL(1),hGIVL(2),hGIVL(3),bp)

!	do i = 1,3,1
!		atot(i) = an(i)+ap(i)
!		btot(i) = bn(i)+bp(i)
!		do j = 1,3,1
!			Rtot(i,j) = Rn(i,j)+Rp(i,j)
!		enddo
!	enddo

!	do k = 1,3,1
!		Flux1L(k) = dt*(atot(k))-dt**2.0/2.0*&
!			(Rtot(k,1)*btot(1)+Rtot(k,2)*btot(2)+Rtot(k,3)*btot(3))
!	enddo
!	print *,Flux1L(1),Flux1L(2),Flux1L(3)

! De-allocate memory
	deallocate(rho0)
	deallocate(mom0)
	deallocate(E0)

	deallocate(rho1)
	deallocate(mom1)
	deallocate(E1)

end program GKSTEST1

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!============================================================================
! Utilities
!============================================================================
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

function step(a_l,a_r,s,x,L)

	real :: x,L,a_l,a_r,s,profile,q,r

    r = (a_l+a_r)/2.0;
    q = (a_r-a_l)/(4.0*atan(1.0));

    step = q*atan(s*(x-L/2.0))+r

    return

end function

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

function peak(a_min,a_max,s,x,L)

	real :: x,L,s,a_max,a_min

    peak = (a_max-a_min)*exp(-s*(x-L/2.0)**2.0)+a_min

    return

end function

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

function erf(x)
	
	real :: erf,t,z,x

!	Returns the error function erf(x) with fractional error everywhere less than
!	1.2  10-7.

	z=abs(x)
	t=1./(1.+0.5*z)
	erf=t*exp(-z*z-1.26551223+t*(1.00002368+t*(.37409196+&
		t*(.09678418+t*(-.18628806+t*(.27886807+t*(-1.13520398+&
		t*(1.48851587+t*(-.82215223+t*.17087277)))))))))
	if (x.lt.0.) erf=2.-erf
	erf = 1.0-erf
	return

end function

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

function average(NV1,NV2)
	
	real :: average,NV1,NV2

	average = (NV1+NV2)/2.0
	return

end function

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

function difference(NV1,NV2)
	
	real :: difference,NV1,NV2

	difference = (NV2-NV1) 
	return

end function

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!============================================================================
! Reconstructions
!============================================================================
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine ReconI(dx,NVL,NVR,IV,GIV)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: dx
	real,dimension(1:3),intent(in) :: NVL
	real,dimension(1:3),intent(in) :: NVR
	real,dimension(1:3),intent(out) :: IV
	real,dimension(1:3),intent(out) :: GIV

	integer :: i
	real :: average,difference

	do i = 1,3,1
		IV(i)  = average(NVL(i),NVR(i))
		GIV(i) = difference(NVL(i),NVR(i))/dx;
	enddo

end subroutine ReconI

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine ReconII(dx,NVW,NVP,NVE,IVLp,IVRn,GIVLp,GIVRn)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: dx
	real,dimension(1:3),intent(in) :: NVW,NVP,NVE
	real,dimension(1:3),intent(out) :: IVLp,IVRn
	real,dimension(1:3),intent(out) :: GIVLp,GIVRn

!		W		|		P		|		E
!				|IVLp		IVRn|		
!				|GIVLp	   GIVRn|
!	--->  x

	real,dimension(1:3) :: sp,sn,S,L
	integer :: k

	do k=1,3,1

		if (-1>0) then
		! Kun Xu
			sp(k) = (NVE(k)-NVP(k))/dx
			sn(k) = (NVP(k)-NVW(k))/dx
			S(k)  = sign(sp(k),0.0)+sign(sn(k),0.0)
		
			if ( (abs(sp(k)) + abs(sn(k)))==0.0 ) then
				L(k)  = 0.0
			else
				L(k)  = S(k)*abs(sp(k))*abs(sn(k))/( abs(sp(k)) + abs(sn(k)) )
			endif

		else
		! Taku Ohwada
			if ( (NVE(k)-NVP(k))*(NVP(k)-NVW(k))>0 ) then
				L(k)=2.0*(NVE(k)-NVP(k))*(NVP(k)-NVW(k))/(dx*(NVE(k)-NVW(k)))
			else
				L(k)=0.0
			endif
		endif

		GIVLp(k) = L(k)
		GIVRn(k) = L(k)
		IVLp(k)  = NVP(k)-L(k)*dx/2.0
		IVRn(k)  = NVP(k)+L(k)*dx/2.0
		
! Reconstruction I
!		GIVLp(k) = (NVP(k)-NVW(k))/dx
!		GIVRn(k) = (NVE(k)-NVP(k))/dx
!		IVLp(k) = (NVP(k)+NVW(k))/2.0
!		IVRn(k) = (NVE(k)+NVP(k))/2.0

	enddo

end subroutine ReconII

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!============================================================================
! Lax-Wendroff
!============================================================================
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine flux(r,mom,E,g,F)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,mom,E,g
	real,dimension(1:3),intent(out) :: F

	F(1) = mom
	F(2) = mom**2.0/r+(g-1.0)*(E-1.0/2.0*mom**2.0/r)
	F(3) = (E+(g-1.0)*(E-1.0/2.0*mom**2.0/r))*mom/r

end subroutine flux

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine matrixLW(r,mom,E,g,M)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,mom,E,g
	real,dimension(1:3,1:3),intent(out) :: M

	M(1,1) = 0.5*(mom**2.0)*(-3.0+g)/(r**2.0)
	M(1,2) = 3.0*mom/r-mom/r*g
	M(1,3) = g-1.0

	M(2,1) = 0.5*mom*(-7.0*mom**2.0+2.0*g*mom**2.0+mom**2.0*g**2.0-2.0*g**2.0*E*r+2.0*g*E*r)/r**3.0
	M(2,2) = 0.5*(12.0*mom**2.0-5*g*mom**2.0-mom**2.0*g**2.0+2.0*g**2.0*E*r-2.0*g*E*r)/r**2.0
	M(2,3) = 3.0*(g-1.0)*mom/r

	M(3,1) = 1.0/4.0*mom**2.0*(8*g*mom**2.0+mom**2.0*g**2.0-9*mom**2.0-6*g*E*r-2.0*g**2.0*E*r)/r**4.0
	M(3,2) = -0.5*mom*(7.0*g*mom**2.0-7.0*mom**2.0-4.0*g*E*r)/r**3.0
	M(3,3) = (g-1.0)*(E+(g-1.0)*E)/r+(-3.0/2.0*(g-1.0)**2.0*mom**2.0+mom**2.0*g**2.0)/r**2.0

end subroutine matrixLW

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
!============================================================================
! GKS
!============================================================================
!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine vector_dp(r,m,E,dp)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E
	real,dimension(1:3),intent(out) :: dp
	real :: A,C,erf

	A = 4.E0/3.E0*E/r-2.E0/3.E0*m**2/r**2
	C = r/sqrt(0.3141593E1*A)**3

	dp(1) = sqrt(A)**3*C*sqrt(0.3141593E1)**3*(1+erf(m/r/sqrt(A)))/2
	
	dp(2) = C*(m*A*sqrt(0.3141593E1*A)+A**2*r*exp(-1/r**2/A*m**2)+&
		erf(m/r/sqrt(A))*m*A*sqrt(0.3141593E1*A))/r*0.3141593E1/2
		
	dp(3) = C*0.3141593E1*sqrt(A)**3*(2*m**2*sqrt(0.3141593E1)+&
		3*sqrt(0.3141593E1)*A*r**2+3*A*sqrt(0.3141593E1)*erf(m/r&
		/sqrt(A))*r**2+2*exp(-1/r**2/A*m**2)*sqrt(A)*m*r+2*m**2*&
		sqrt(0.3141593E1)*erf(m/r/sqrt(A)))/r**2/8

end subroutine vector_dp

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine vector_ap(r,m,E,ap)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E
	real,dimension(1:3),intent(out) :: ap
	real :: erf

	ap(1) = sqrt(2.E0)*sqrt(3.E0)*(m*sqrt(2.E0)*sqrt(2*E*r-m**2)*&
		sqrt(0.3141593E1)*sqrt(3.E0)+4*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*E*r&
		-2*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*m**2+erf(m/r*sqrt(6.E0)/&
		sqrt((2*E*r-m**2)/r**2)/2)*m*sqrt(2.E0)*sqrt(2*E*r-m**2)*&
		sqrt(0.3141593E1)*sqrt(3.E0))/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)/12

	ap(2) = (2*m**2*sqrt(0.3141593E1)+2*sqrt(0.3141593E1)*E*r+&
		exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*m&
		+2*m**2*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)&
		+2*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*&
		E*r)/sqrt(0.3141593E1)/r/6

	ap(3) = (30*m*sqrt(0.3141593E1)*E*r-6*m**3*sqrt(0.3141593E1)&
		+30*m*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*E*r&
		-6*m**3*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)&
		-exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*&
		m**2+8*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*&
		sqrt(3.E0)*E*r)/r**2/sqrt(0.3141593E1)/36

end subroutine vector_ap

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine matrix_Rp(r,m,E,Rp)

	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E
	real,dimension(1:3,1:3),intent(out) :: Rp
	real :: erf

	Rp(1,1) = exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)/sqrt(2*E*r-m**2)*&
		sqrt(2.E0)*E/sqrt(0.3141593E1)/6
	Rp(1,2) = (6*sqrt(0.3141593E1)*E*r+6*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/&
		sqrt((2*E*r-m**2)/r**2)/2)*E*r-exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*m-3*m**2*sqrt(0.3141593E1)*&
		erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)-3*m**2*sqrt(0.3141593E1))/&
		(2*E*r-m**2)/sqrt(0.3141593E1)/6
	Rp(1,3) = exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)/sqrt(2*E*r-m**2)*&
		sqrt(3.E0)*r/sqrt(0.3141593E1)/6

	Rp(2,1) = -(2*sqrt(2.E0)*sqrt(3.E0)*E*r-m**2*sqrt(2.E0)*sqrt(3.E0)+2*m*&
		sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2))&
		+2*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*m*sqrt(0.3141593E1)*&
		sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2)))*m/exp(3.E0/2.E0*&
		m**2/(2*E*r-m**2))/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)/r**2/6
	Rp(2,2) = (2*sqrt(2.E0)*sqrt(3.E0)*E*r-m**2*sqrt(2.E0)*sqrt(3.E0)+2*m*&
		sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2))&
		+2*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*m*sqrt(0.3141593E1)*&
		sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2)))/exp(3.E0/2.E0*&
		m**2/(2*E*r-m**2))/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)/r/3
	Rp(2,3) = 1.E0/3.E0+erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)/3

	Rp(3,1) = (-15*r*E*m*sqrt(0.3141593E1)-2*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*r*E-15*r*E*sqrt(0.3141593E1)*&
		m*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)+6*m**3*sqrt(0.3141593E1)&
		+6*sqrt(0.3141593E1)*m**3*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)&
		+exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*&
		m**2)/sqrt(0.3141593E1)/r**3/18
	Rp(3,2) = (10*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*&
		E*r+10*sqrt(0.3141593E1)*E*r-exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*&
		sqrt(2*E*r-m**2)*sqrt(3.E0)*m-6*m**2*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/&
		sqrt((2*E*r-m**2)/r**2)/2)-6*m**2*sqrt(0.3141593E1))/sqrt(0.3141593E1)/r**2/12
	Rp(3,3) = (5*m*sqrt(0.3141593E1)+2*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)*&
		sqrt(2.E0)*sqrt(2*E*r-m**2)+5*sqrt(0.3141593E1)*m*erf(m/r*sqrt(6.E0)/&
		sqrt((2*E*r-m**2)/r**2)/2))/r/sqrt(0.3141593E1)/6

end subroutine matrix_Rp

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine vector_bp(r,m,E,Gr,Gm,GE,bp)

	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E,Gr,Gm,GE
	real,dimension(1:3),intent(out) :: bp
	real :: erf,s1,s2,s3,s4,s5

	bp(1) = (6*r*E*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*Gm&
		+6*r*E*sqrt(0.3141593E1)*Gm+r*sqrt(3.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(2.E0)*GE*sqrt(2*E*r-m**2)+sqrt(3.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(2.E0)*Gr*sqrt(2*E*r-m**2)*E-3*sqrt(0.3141593E1)*m**2*Gm-3*m**2*&
		sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*Gm-sqrt(3.E0)*&
		exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*m*Gm*sqrt(2*E*r-m**2))/(2*E*r-m**2)/&
		sqrt(0.3141593E1)/6

	s2 = 1.E0/6.E0
	s5 = 4*r**2*Gm*sqrt(2.E0)*sqrt(3.E0)*E+2*GE*r**2*sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*&
		exp(3.E0/2.E0*m**2/(2*E*r-m**2))+2*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*&
		GE*r**2*sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2))-2*r*m*&
		Gr*sqrt(2.E0)*sqrt(3.E0)*E+4*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*Gm*m*r*&
		sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2))      
	s4 = s5+4*Gm*m*r*sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2))&
		-2*r*m**2*Gm*sqrt(2.E0)*sqrt(3.E0)+m**3*Gr*sqrt(2.E0)*sqrt(3.E0)-2*Gr*m**2*&
		sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*exp(3.E0/2.E0*m**2/(2*E*r-m**2))-2*erf(m/r*&
		sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*Gr*m**2*sqrt(0.3141593E1)*sqrt(2*E*r-m**2)*&
		exp(3.E0/2.E0*m**2/(2*E*r-m**2))      
	s5 = 1/r**2      
	s3 = s4*s5      
	s1 = s2*s3      
	s2 = 1/sqrt(0.3141593E1)/sqrt(2*E*r-m**2)/exp(3.E0/2.E0*m**2/(2*E*r-m**2))      
	bp(2) = s1*s2

	s2 = 5.E0/6.E0*r**2*E*sqrt(0.3141593E1)*Gm*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)&
		+5.E0/6.E0*r**2*E*sqrt(0.3141593E1)*Gm+5.E0/6.E0*r**2*m*sqrt(0.3141593E1)*erf(m/r*&
		sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*GE+sqrt(2.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(3.E0)*GE*sqrt(2*E*r-m**2)*r**2/3+5.E0/6.E0*r**2*sqrt(0.3141593E1)*m*GE&
		-r*sqrt(2.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)*Gr*E*sqrt(2*E*r-m**2)/9&
		-5.E0/6.E0*r*E*sqrt(0.3141593E1)*m*Gr      
	s1 = s2-5.E0/6.E0*r*E*m*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*Gr&
		-r*sqrt(2.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)*m*Gm*sqrt(2*E*r-m**2)/12&
		-r*sqrt(0.3141593E1)*m**2*Gm/2-r*m**2*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r&
		-m**2)/r**2)/2)*Gm/2+m**3*sqrt(0.3141593E1)*erf(m/r*sqrt(6.E0)/sqrt((2*E*r-m**2)/r**2)/2)*&
		Gr/3+sqrt(2.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)*Gr*m**2*sqrt(2*E*r-m**2)/18&
		+sqrt(0.3141593E1)*m**3*Gr/3
	s2 = 1/sqrt(0.3141593E1)/r**3
	bp(3) = s1*s2

end subroutine vector_bp

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine vector_dn(r,m,E,dn)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E
	real,dimension(1:3),intent(out) :: dn
	real :: A,C,erf

	A = 4.E0/3.E0*E/r-2.E0/3.E0*m**2/r**2
	C = r/sqrt(0.3141593E1*A)**3

	dn(1) = -sqrt(A)**3*C*sqrt(0.3141593E1)**3*(erf(m/r/sqrt(A))-1)/2
	
	dn(2) = -C*(A**2*r*exp(-1/r**2/A*m**2)+erf(m/r/sqrt(A))*m*A*&
		sqrt(0.3141593E1*A)-m*A*sqrt(0.3141593E1*A))/r*0.3141593E1/2
		
	dn(3) = -C*0.3141593E1*sqrt(A)**3*(2*exp(-1/r**2/A*m**2)*sqrt(A)*&
		m*r+2*m**2*sqrt(0.3141593E1)*erf(m/r/sqrt(A))+3*A*sqrt(0.3141593E1)*&
		erf(m/r/sqrt(A))*r**2-2*m**2*sqrt(0.3141593E1)-3*A*sqrt(0.3141593E1)*r**2)/r**2/8

end subroutine vector_dn

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine vector_an(r,m,E,an)
	
	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E
	real,dimension(1:3),intent(out) :: an
	real :: erf

	an(1) = m/2-sqrt(2.E0)*sqrt(3.E0)/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)*exp(-3.E0/2.E0*m**2/&
		(2*E*r-m**2))*E*r/3+sqrt(2.E0)*sqrt(3.E0)/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)*&
		exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*m**2/6-erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*m/2
		
	an(2) = 1/r*m**2/3+E/3-1/sqrt(0.3141593E1)/r*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*&
		sqrt(2*E*r-m**2)*sqrt(3.E0)*m/6-1/r*m**2*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)/3&
		-erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*E/3      

	an(3) = -m**3/r**2/6+5.E0/6.E0*m/r*E-5.E0/6.E0/r*m*erf(m/r*sqrt(6.E0)/&
		sqrt(2*E/r-m**2/r**2)/2)*E+1/r**2*m**3*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)/6&
		+1/r**2/sqrt(0.3141593E1)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*&
		sqrt(2*E*r-m**2)*sqrt(3.E0)*m**2/36-2.E0/9.E0/r/sqrt(0.3141593E1)*&
		exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*E

end subroutine vector_an


!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine matrix_Rn(r,m,E,Rn)

	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E
	real,dimension(1:3,1:3),intent(out) :: Rn
	real :: erf

	Rn(1,1) = -sqrt(2.E0)*sqrt(3.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*E/&
		sqrt(2*E*r-m**2)/sqrt(0.3141593E1)/6
	Rn(1,2) = 1-1/(2*E*r-m**2)*E*r-1/(2*E*r-m**2)*erf(m/r*sqrt(6.E0)/&
		sqrt(2*E/r-m**2/r**2)/2)*E*r+1/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)*&
		exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(3.E0)*m/6+1/(2*E*r&
		-m**2)*m**2*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)/2+m**2/(2*E*r-m**2)/2
	Rn(1,3) = -exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)/sqrt(2*E*r-m**2)*&
		sqrt(3.E0)*r/sqrt(0.3141593E1)/6
	
	Rn(2,1) = -m**2/r**2/3+m/exp(3.E0/2.E0*m**2/(2*E*r-m**2))/sqrt(2*E*r-m**2)/&
		sqrt(0.3141593E1)/r*sqrt(2.E0)*sqrt(3.E0)*E/3-m**3/exp(3.E0/2.E0*m**2/&
		(2*E*r-m**2))/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)/r**2*sqrt(2.E0)*sqrt(3.E0)/6&
		+m**2/r**2*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)/3
	Rn(2,2) = 2.E0/3.E0/r*m-2.E0/3.E0/exp(3.E0/2.E0*m**2/(2*E*r-m**2))/sqrt(2*E*r-m**2)/&
		sqrt(0.3141593E1)*sqrt(2.E0)*sqrt(3.E0)*E+1/exp(3.E0/2.E0*m**2/(2*E*r-m**2))/&
		sqrt(2*E*r-m**2)/sqrt(0.3141593E1)/r*m**2*sqrt(2.E0)*sqrt(3.E0)/3-2.E0/3.E0/r*&
		erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*m
	Rn(2,3) = 1.E0/3.E0-erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)/3

	Rn(3,1) = m**3/r**3/3-5.E0/6.E0*m/r**2*E+1/sqrt(0.3141593E1)/r**2*exp(-3.E0/2.E0*m**2/&
		(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*E/9+5.E0/6.E0/r**2*m*&
		erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*E-1/r**3*m**3*erf(m/r*sqrt(6.E0)/&
		sqrt(2*E/r-m**2/r**2)/2)/3-1/sqrt(0.3141593E1)/r**3*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(2.E0)*sqrt(2*E*r-m**2)*sqrt(3.E0)*m**2/18
	Rn(3,2) = -m**2/r**2/2+5.E0/6.E0*E/r-5.E0/6.E0/r*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*E&
		+1/sqrt(0.3141593E1)/r**2*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*sqrt(2*E*r-m**2)*&
		sqrt(3.E0)*m/12+m**2/r**2*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)/2
	Rn(3,3) = 5.E0/6.E0/r*m-1/r/sqrt(0.3141593E1)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)*&
		sqrt(2.E0)*sqrt(2*E*r-m**2)/3-5.E0/6.E0/r*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*m

end subroutine matrix_Rn

!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&

subroutine vector_bn(r,m,E,Gr,Gm,GE,bn)

	implicit none

!	Inlet / Outlet
	real,intent(in) :: r,m,E,Gr,Gm,GE
	real,dimension(1:3),intent(out) :: bn
	real :: erf,s1

	bn(1) = Gm-1/(2*E*r-m**2)*r*E*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*Gm-1/(2*E*r-m**2)*r*&
		E*Gm-1/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)*r*sqrt(3.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(2.E0)*GE/6-1/sqrt(2*E*r-m**2)/sqrt(0.3141593E1)*sqrt(3.E0)*exp(-3.E0/2.E0*m**2/&
		(2*E*r-m**2))*sqrt(2.E0)*Gr*E/6+1/(2*E*r-m**2)*m**2*Gm/2+1/(2*E*r-m**2)*m**2*&
		erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*Gm/2+1/sqrt(2*E*r-m**2)/&
		sqrt(0.3141593E1)*sqrt(3.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(2.E0)*m*Gm/6
	
	bn(2) = -1/r**2*Gr*m**2/3+2.E0/3.E0/r*Gm*m+GE/3-2.E0/3.E0/sqrt(0.3141593E1)/&
		sqrt(2*E*r-m**2)/exp(3.E0/2.E0*m**2/(2*E*r-m**2))*Gm*sqrt(2.E0)*sqrt(3.E0)*E&
		-erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*GE/3+1/r/sqrt(0.3141593E1)/&
		sqrt(2*E*r-m**2)/exp(3.E0/2.E0*m**2/(2*E*r-m**2))*m*Gr*sqrt(2.E0)*sqrt(3.E0)*&
		E/3-2.E0/3.E0/r*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*Gm*m+1/r/&
		sqrt(0.3141593E1)/sqrt(2*E*r-m**2)/exp(3.E0/2.E0*m**2/(2*E*r-m**2))*m**2*&
		Gm*sqrt(2.E0)*sqrt(3.E0)/3-1/r**2/sqrt(0.3141593E1)/sqrt(2*E*r-m**2)/&
		exp(3.E0/2.E0*m**2/(2*E*r-m**2))*m**3*Gr*sqrt(2.E0)*sqrt(3.E0)/6+1/r**2*&
		erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*Gr*m**2/3

	s1 = 1/r**3*Gr*m**3/3-1/r**2*Gm*m**2/2+5.E0/6.E0/r*m*GE-5.E0/6.E0/r**2*m*Gr*E&
		+5.E0/6.E0/r*Gm*E-5.E0/6.E0/r*E*Gm*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)&
		-5.E0/6.E0/r*m*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*GE      
	bn(3) = s1-1/sqrt(0.3141593E1)/r*sqrt(2.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(3.E0)*GE*sqrt(2*E*r-m**2)/3+1/sqrt(0.3141593E1)/r**2*sqrt(2.E0)*&
		exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*sqrt(3.E0)*Gr*E*sqrt(2*E*r-m**2)/9&
		+5.E0/6.E0/r**2*E*m*erf(m/r*sqrt(6.E0)/sqrt(2*E/r-m**2/r**2)/2)*Gr&
		+1/sqrt(0.3141593E1)/r**2*sqrt(2.E0)*exp(-3.E0/2.E0*m**2/(2*E*r-m**2))*&
		sqrt(3.E0)*m*Gm*sqrt(2*E*r-m**2)/12+1/r**2*m**2*erf(m/r*sqrt(6.E0)/&
		sqrt(2*E/r-m**2/r**2)/2)*Gm/2-1/r**3*m**3*erf(m/r*sqrt(6.E0)/sqrt(2*E/r&
		-m**2/r**2)/2)*Gr/3-1/sqrt(0.3141593E1)/r**3*sqrt(2.E0)*exp(-3.E0/2.E0*&
		m**2/(2*E*r-m**2))*sqrt(3.E0)*Gr*m**2*sqrt(2*E*r-m**2)/18

end subroutine vector_bn
